package org.firstinspires.ftc.teamcode.subsystem;

import static org.firstinspires.ftc.teamcode.configs.RobotConstants.liftPower;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.liftToHighBucketPos;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.liftToLowBucketPos;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.liftZeroPos;

import com.qualcomm.robotcore.hardware.DcMotor;
import com.qualcomm.robotcore.hardware.DcMotorSimple;
import com.qualcomm.robotcore.hardware.HardwareMap;

import org.firstinspires.ftc.teamcode.util.action.RunAction;

public class LiftSubsystem {

    public DcMotor lift;
    public RunAction toZero, toLowBucket, toHighBucket;
    public enum LiftState {
        FLOOR, LOW_BUCKET, HIGH_BUCKET
    }
    private LiftState liftState;

    public LiftSubsystem(HardwareMap hardwareMap) {
        lift = hardwareMap.get(DcMotor.class, "lift-motor");

        toZero = new RunAction(this::toZero);
        toLowBucket = new RunAction(this::toLowBucket);
        toHighBucket = new RunAction(this::toHighBucket);
    }

    public void setTarget(int b) {
        lift.setTargetPosition(b);
        lift.setMode(DcMotor.RunMode.RUN_TO_POSITION);
    }

    public void toZero() {
        lift.setTargetPosition(liftZeroPos);
        lift.setMode(DcMotor.RunMode.RUN_TO_POSITION);
    }

    public void switchState() {
        if(this.liftState == LiftState.FLOOR) {
            this.toLowBucket();
        } else if (this.liftState == LiftState.LOW_BUCKET) {
            this.toHighBucket();
        } else if (this.liftState == LiftState.HIGH_BUCKET) {
            this.toZero();
        }
    }

    public void toLowBucket() {
        lift.setTargetPosition(liftToLowBucketPos);
        lift.setMode(DcMotor.RunMode.RUN_TO_POSITION);
    }

    public void toHighBucket() {
        lift.setTargetPosition(liftToHighBucketPos);
        lift.setMode(DcMotor.RunMode.RUN_TO_POSITION);
    }

    public void init() {
        lift.setMode(DcMotor.RunMode.STOP_AND_RESET_ENCODER);
        lift.setDirection(DcMotorSimple.Direction.REVERSE);
        lift.setPower(liftPower);
    }

    private void setState(LiftState liftState) {
        this.liftState = liftState;
    }

    public int getPosition() {
        return lift.getCurrentPosition();
    }

    public void start() {
        setTarget(10);
    }

}
